// ----------------------------------------------------------------------------
// Exams.java
// analyzing the grades of exams
// 
// Author:  Wolfgang Schreiner <Wolfgang.Schreiner@fhs-hagenberg.ac.at>
// Created: August 24, 2001
// ----------------------------------------------------------------------------
public class Exams3
{
  // --------------------------------------------------------------------------
  // analyze the grades of multiple exams; read the grades from the standard
  // input stream, writes the result to the standard output stream.
  // for details, see 'analyzeExamGrades'.
  // --------------------------------------------------------------------------
  public static void main(String args[])
  {
    analyzeExamGrades();
  }

  // --------------------------------------------------------------------------
  // user repeatedly enters grades from 1 to 5 or 0 to denote the end of exam.
  // system prints number of grades, best, worst and average grade and
  // asks whether to proceed with another exam.
  // --------------------------------------------------------------------------
  public static void analyzeExamGrades()
  {
    char answer = 0;
    do
    {
      Result2 result = analyzeExam();
      result.print();
      answer = askToContinue();
    }
    while (answer == 'y');
  }

  // --------------------------------------------------------------------------
  // iteratively reads and analyzes the grades of an exam.
  // when a grade 0 is read, the result of the analysis is returned.
  // --------------------------------------------------------------------------
  public static Result2 analyzeExam()
  {
    Result2 result = new Result2();

    // read and process exam
    while (true)
    {
      int grade = readGrade(result.getCount());
      if (grade == 0) return result;
      result.process(grade);
    }
  }

  // --------------------------------------------------------------------------
  // asks the user for grade number 'count' (starting with 0).
  // if the user enters an integer number from 0 to 5, this is returned as
  // the result. otherwise, the user is asked again.
  // aborts execution, if the input stream has terminated.
  // --------------------------------------------------------------------------
  public static int readGrade(int count)
  {
    while(true)
    {
      System.out.print("Enter grade "+(count+1)+" (0 when done): ");
      int grade = Input.readInt();
      if (Input.isOkay() && 0 <= grade && grade <= 5)
        return grade;
      if (Input.hasEnded())
      {
        System.out.println("Unexpected end of input!");
        System.exit(-1);
      }
      System.out.println("Invalid input!");
      Input.clearError();
    }
  }

  // --------------------------------------------------------------------------
  // asks user whether to continue. if the user answers 'y' or 'n', this
  // is returned as the result. otherwise, the user is asked again.
  // aborts execution, if the input stream has terminated.
  // --------------------------------------------------------------------------
  public static char askToContinue()
  {    
    while(true)
    {
      System.out.print("Another exam (y/n)? ");             
      char answer = Input.readChar();
      if (Input.isOkay() && (answer == 'y' || answer == 'n'))
        return answer;
      if (Input.hasEnded())
      {
        System.out.println("Unexpected end of input!");
        System.exit(-1);
      }
      System.out.println("Invalid input!");
    }      
  }
}
